/**
 * @hidden
 * @packageDocumentation
 */
import { SemanticError } from "../../standard-edition/model/errors";
import { UndoRedoEditor, UndoRedoEditorState } from "../../standard-edition/editor/undo-redo-editor";
import { MathStructure, MathSymbol } from "./model";
import { NavDirection, Stateful, HorizontalNavDirection } from "../../standard-edition/editor/editor";
import { Terminal } from "../../standard-edition/model/model-base";
import { Renderer, RenderedTree } from "../../standard-edition/editor/editor-support";
import { UpDownHandler } from "../../standard-edition/editor/up-down-handler";
import { ConfirmRequiredOperation } from "../../standard-edition/engine/internal-engine";
import { HorizontalDirection, VerticalDirection } from "../../standard-edition/engine/utils";
import { SemanticAdviceManager } from "./advice/semantic-advice-manager";
import { IAdvice } from "../../standard-edition/accessibility/advices";
import { NotifyObject, NotifyUpdateObject, TreeUpdateCallbacks } from "../../standard-edition/dom/dombinder-notify-updater";
import { DombinderA11ySpeaker } from "../../standard-edition/dom/dombinder-a11y-speaker";
import { SemanticDombinderNotifyUpdater } from "./semantic-dombinder-notify-updater";
import { MathNodeValidationVisitor } from "./util/validation";
import { Heuristic } from "./util/heuristics";
import { MathTreeSettings } from "./util/utils";
import { CursorNode, CursorSide } from "../../standard-edition/dom/mouse-selection";
import { KatexNodeRect } from "../../standard-edition/renderer/katex/types";
/**
 * It is the base class of the editor's class hierarchy.
 * It allows creation of the editor in memory and communicating with it via API.
 * It does not have ability to render. Usually it is used for the cases when one needs some calculation with the editor itself or some of its plugins.
 * Also it is useful for testing as it does not depend on DOM.
 * Use {@link createAPI} function to create an instance of this class.
 * @category API
 */
export declare class MathEditorAPI {
    /**
     * @hidden
     */
    readonly editor: UndoRedoEditor & Stateful<UndoRedoEditorState>;
    heuristics: Heuristic[];
    /**
     * @hidden
     */
    readonly updateNotifier: SemanticDombinderNotifyUpdater;
    /**
     * @hidden
     */
    readonly speaker: DombinderA11ySpeaker;
    /**
     * @hidden
     */
    readonly semanticValidationVisitor: MathNodeValidationVisitor;
    /**
     * @hidden
     */
    protected readonly adviceManager: SemanticAdviceManager;
    /**
     * @hidden - external user should never instantiate this class via constructor.
     */
    constructor(callbacks?: TreeUpdateCallbacks<ExtendedNotifyObject, NotifyUpdateObject>);
    /**
     * Applies all the previously installed heuristics.
     * @returns true if the heuristics have changed the current expression, false if not.
     * @category API
     */
    applyHeuristics(): boolean;
    /**
     * @returns representation of current cursor
     * See {@link Cursor} for details.
     * @category API
     */
    getCursor(): Cursor;
    /**
     * Places cursor at the left or at the right of the given symbol
     * @param nodeId - identifier of the given symbol (can be found in model classes)
     * @param side - where to place the cursor - at the right of the symbol or at the left
     * @category API
     */
    placeCursor(nodeId: number, side: CursorSide): boolean;
    /**
     * Removes cursor from the editor
     * @returns true if the cursor was actually in the editor
     * @category API
     */
    removeCursor(): boolean;
    /**
     * Returns symbols located near the cursor.
     * @param includeHidden - include hidden symbols (such as implicit multiplication) or not
     * @category API
     * @gradarius
     */
    getSymbolsAtCursor(includeHidden?: boolean): number[];
    /**
     * Add listener to listen for all the editor updates.
     * @param cb
     * The listener will receive two parameters:
     * updateInfo - what exactly were updated
     * and
     * confirmation - what was not yet updated, but the user was asked to confirm the operation instead
     * @category API
     */
    addOnUpdate(cb: (updateInfo: NotifyUpdateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    /**
     * Remove the listener previously added with {@link addOnUpdate}.
     * @param cb - the listener to remove
     * @category API
     */
    removeOnUpdate(cb: (updateInfo: NotifyUpdateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    /**
     * Allows listening to all the events that do not change the editor's model: neither change expression nor cursor nor selection.
     * @param cb - the listener to add
     * @category API
     */
    addOnNotify(cb: (updateInfo: ExtendedNotifyObject[]) => void): void;
    /**
     * Remove the listener previously added with {@link addOnNotify}.
     * @param cb - the listener to remove
     * @category API
     */
    removeOnNotify(cb: (updateInfo: ExtendedNotifyObject[]) => void): void;
    /**
     * @hidden
     */
    pauseSync(): void;
    /**
     * @hidden
     */
    resumeSync(insideOnUpdate: boolean): void;
    /**
     * Returns advices for the current cursor position. Advice is a suggestion to end user what can be inserted into the current cursor position.
     * For example, if user enters 'in' then we can suggest him 'Indefinite integral'.
     * @category API
     */
    getAdvices(): IAdvice[];
    /**
     * Applies advice received from the {@link getAdvices} method.
     * @category API
     */
    applyAdvice(advice: IAdvice): this;
    /**
     * Returns description of the current expression as human readable math text.
     * @category API
     */
    getSemanticSpeech(): string;
    /**
     * Returns description of the current expression as character-by-character text.
     * @category API
     */
    getCharacterSpeech(): string;
    /**
     * @hidden
     */
    toggleInputSymbolsReading(value: boolean): void;
    /**
     * @returns current expression as latex
     * @category API
     */
    getLatex(): string;
    /**
     * @returns current expression in context free semantic language
     * @category API
     */
    getSemanticText(): string;
    /**
     * @returns current expression as semantic tree with the list of semantic errors (if any)
     * @category API
     */
    getSemantic(): {
        semantic: MathStructure;
        errors: SemanticErrorDescription[];
    };
    /**
     * @returns current expression as semantic tree
     * @param settings - allow to post process semantic tree to omit information (eg selection, cursor, etc)
     * @category API
     */
    getTree(settings?: MathTreeSettings): MathStructure;
    /**
     * Updates current expression with the given semantic tree
     * @param tree - semantic tree
     * @returns updated math tree (original one can be changed by hints)
     * @category API
     */
    setTree(tree: MathStructure): MathStructure;
    /**
     * Inserts character to the right of the current cursor position or replaces the selection.
     * This method allows programmatically emulate user input.
     * It requires cursor or non-empty selection or empty expression inside the editor. Otherwise, it does nothing.
     * @returns true if the character was successfully inserted
     * @param s - character to insert
     * @category API
     */
    insertSymbol(s: string): boolean;
    /**
     * Inserts tree to the right of the current cursor position or replaces the selection.
     * This method allows programmatically to insert one semantic tree into another.
     * It requires cursor or non-empty selection or empty expression inside the editor. Otherwise, it does nothing.
     * @returns true if the tree was successfully inserted
     * @param tree - semantic tree to insert
     * @param overrideContext
     *  if true then overrides semantic context of the insertable tree to match current one (default)
     *  if false then throws an exception on context conflicts
     * @category API
     */
    insertTree(tree: MathStructure, overrideContext?: boolean): boolean;
    /**
     * Copies selected expression to the clipboard.
     * @returns - copied expression
     * @category API
     */
    copy(): string;
    /**
     * Cuts selected expression to the clipboard.
     * @returns - cut expression
     * @category API
     */
    cut(): string;
    /**
     * Pastes expression from the clipboard to the right of the current cursor position or replaces the selection.
     * It requires cursor or non-empty selection or empty expression inside the editor. Otherwise, it does nothing.
     * @returns true if the expression was successfully pasted
     * @category API
     */
    paste(latex: string): boolean;
    /**
     * Programmatically increases selection in a given direction.
     * It requires cursor or non-empty selection inside the editor.
     * @param direction - direction to increase the selection
     * @category API
     */
    select(direction: NavDirection): MathEditorAPI;
    /**
     * Programmatically creates selection between given nodes using the provided horizontal and vertical direction.
     * @param startNodeId - start node id
     * @param endNodeId - end node id
     * @param horizontalDirection - horizontal direction
     * @param verticalDirection - vertical direction
     * @param includeStartNode - whether include start node to selection or not (default true)
     * @category API
     * @gradarius
     */
    selectByNodeIds(startNodeId: number, endNodeId: number, horizontalDirection: HorizontalDirection, verticalDirection: VerticalDirection, includeStartNode?: boolean): MathEditorAPI;
    /**
     * Programmatically moves the selection boundary horizontally.
     * @param leftBoundary - whether move left or right boundary
     * @param direction - direction to move selection boundary
     * @category API
     */
    moveSelectionHorizontalBoundary(leftBoundary: boolean, direction: HorizontalDirection): MathEditorAPI;
    /**
     * Programmatically moves the selection boundary vertically.
     * @param topBoundary - whether move top or bottom boundary
     * @param direction - direction to move selection boundary
     * @category API
     */
    moveSelectionVerticalBoundary(topBoundary: boolean, direction: VerticalDirection): MathEditorAPI;
    /**
     * @returns current selection as semantic tree
     * @category API
     */
    getSelectionTree(): MathStructure;
    /**
     * Returns whether selection exists or not.
     * @category API
     */
    hasSelection(): boolean;
    /**
     * @hidden
     * @gradarius
     */
    getSelectionHorizontalDirection(): HorizontalDirection | null;
    /**
     * Drops the current selection.
     * @param placeCursorAtSelectionEnd - whether put the cursor to the end of selection after dropping
     * @category API
     */
    dropSelection(placeCursorAtSelectionEnd: boolean): void;
    /**
     * @hidden
     * @gradarius
     */
    getBoundaryTerminal(left: boolean, onlyVisible?: boolean): number;
    /**
     * @hidden
     * @gradarius
     */
    getBoundaryPointer(left: boolean, onlyVisible?: boolean): CursorNode;
    /**
     * @hidden
     * @gradarius
     */
    getSelectionBoundaryTerminal(horizontalDirection: HorizontalDirection, verticalDirection?: VerticalDirection): number;
    /**
     * @hidden
     * @gradarius
     */
    getSelectionBoundaryPointer(horizontalDirection: HorizontalDirection, verticalDirection?: VerticalDirection): CursorNode;
    /**
     * @category API
     */
    undo(): MathEditorAPI;
    /**
     * @category API
     */
    redo(): MathEditorAPI;
    /**
     * Returns internal editor state as JSON.
     */
    getCurrentState(): string;
    /**
     * Sets internal editor state using provided JSON.
     * @param state - internal editor state
     */
    setCurrentState(state: string): void;
    /**
     * Returns internal editor state with history as JSON.
     */
    getCurrentStateWithHistory(): string;
    /**
     * Sets internal editor state with history using provided JSON.
     * @param state - internal editor state
     */
    setCurrentStateWithHistory(state: string): void;
    /**
     * Returns internal expression representation.
     */
    getExpressionInternalRepresentation(): string;
    /**
     * Sets internal expression representation.
     * Returns an array of node ids, ordered left to right according to initial order in the math expression.
     * @param mathExpression - internal expression representation
     */
    setExpressionInternalRepresentation(mathExpression: string): number[];
    /**
     * Deletes node near the current cursor position or replaces the selection.
     * This method allows programmatically emulate pressing delete/backspace button.
     * It requires cursor or non-empty selection inside the editor. Otherwise, it does nothing.
     * @param direction - left for backspace button and right for delete button
     * @param force - whether node or ask user for confirmation
     * @category API
     */
    delete(direction: HorizontalDirection, force?: boolean): this;
    /**
     * Programmatically moves cursor to the given direction.
     * It requires cursor inside the editor. Otherwise, it does nothing.
     * @param direction - direction to move cursor to
     * @category API
     */
    navigate(direction: HorizontalNavDirection): boolean;
    /**
     * Makes editor readonly and creates editable cells inside.
     * @param editableCells
     *  nodeId - node id for editable cell
     *  borderVisible - whether the border of editable cell visible or not
     *  avoidParenthesesWrap - whether the wrap editable cell with parens or not
     * @category API
     */
    defineEditableCell(editableCells: {
        nodeId: number;
        borderVisible?: boolean;
        avoidParenthesesWrap?: boolean;
    }[]): MathEditorAPI;
    /**
     * @hidden
     */
    protected notifyOnUpdate(updateType: NotifyUpdateObject | NotifyUpdateObject[], confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean): void;
    /**
     * @hidden
     */
    protected notify(updateType: ExtendedNotifyObject | ExtendedNotifyObject[], confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean): void;
}
/**
 * Event that represents semantic event (for example validation).
 * @category API
 */
export declare enum SemanticNotifyObject {
    validation = "validation"
}
export type ExtendedNotifyObject = SemanticNotifyObject | NotifyObject;
interface CustomDecoration {
    nodeIds: number[];
    painter: (element: HTMLElement) => void;
    omitRedraw?: boolean;
    preDelete?: () => void;
    guid: string;
}
/**
 * Extends {@link MathEditorAPI} and renders the expression using using provided renderer.
 * Allows to compute boundaries and render decorations of the expression and its parts.
 * Use {@link createRenderableEditor} function to create an instance of this class.
 * @category API
 */
export declare class MathEditorRenderable extends MathEditorAPI {
    /**
     * @hidden
     */
    protected renderedTree: RenderedTree;
    /**
     * @hidden
     */
    protected readonly renderer: Renderer;
    /**
     * @hidden
     */
    protected decorations: CustomDecoration[];
    /**
     * @hidden
     */
    protected readonly upDown: UpDownHandler;
    /**
     * @hidden - external user should never instantiate this class via constructor.
     */
    constructor(renderer: Renderer, callbacks?: TreeUpdateCallbacks<ExtendedNotifyObject, NotifyUpdateObject>);
    /**
     * Navigates cursor vertically inside the editor.
     * @param {VerticalDirection} direction
     * @return {boolean} true if navigation happened (there was somewhere to go)
     */
    navigateVertically(direction: VerticalDirection): boolean;
    /**
     * Decorates math nodes inside the editor. For that, it creates an overlay element you can style.
     * The decoration is dropped if the tree changed because it becomes disconnected with new tree nodes.
     * @param nodeIds - node ids to decorate
     * @param decorator - function to decorate the overlay element
     * @param preDelete - hook that executes when the decoration is removed on math tree update
     * @param withRender - indicates if the decoration requires rendering after creation
     * @param omitRedraw -  indicates if the decoration requires redrawing
     * @returns handler to remove the decoration
     */
    decorateNodes(nodeIds: number[], decorator: (element: HTMLElement) => void, preDelete?: () => void, withRender?: boolean, omitRedraw?: boolean): () => void;
    /**
     * Re-renders all decorations.
     */
    renderDecorationsOnly(): void;
    /**
     * Removes all decorations.
     */
    cleanDecorations(): void;
    /**
     * Clears the editor and remove all decorations.
     */
    cleanEditorWithoutNotify(): void;
    /**
     * Returns ClientRect for the current cursor.
     */
    getCursorBoundingRect(): KatexNodeRect;
    /**
     * Returns ClientRect for the current selection.
     */
    getSelectionBoundingRect(): KatexNodeRect[];
    /**
     * @hidden
     */
    protected render(decorationOnly?: boolean): void;
    /**
     * @hidden
     */
    protected getTerminals(nodeId: number): Terminal[];
}
/**
 * Represents cursor inside the editor.
 * @field leftNode - node to the left of the cursor
 * @field rightNode - node to the right of the cursor
 * @field atBaseLevel - whether cursor at base level or not
 * @category API
 */
export declare class Cursor {
    readonly leftNode: MathSymbol;
    readonly rightNode: MathSymbol;
    readonly anyNode: MathSymbol;
    readonly anySide: CursorSide;
    readonly atBaseLevel: boolean;
    constructor(leftNode: MathSymbol, rightNode: MathSymbol, atBaseLevel: boolean);
    get isEmpty(): boolean;
}
/**
 * Represents error description.
 * @field uuid - unique identifier
 * @field nodeIds - affected node ids
 * @field description - human readable description
 * @category API
 */
export declare class SemanticErrorDescription {
    readonly uuid: string;
    readonly nodeIds: number[];
    readonly description: string;
    constructor(nodeIds: number[], description: string);
}
export declare function convertSemanticError(semanticError: SemanticError): SemanticErrorDescription;
/**
 * Factory method that creates {@link MathEditorAPI}.
 * @category API
 */
export declare function createAPI(): MathEditorAPI;
/**
 * Factory method that creates {@link MathEditorRenderable} using given renderer. Currently there is only one renderer that uses Katex.
 * @category API
 */
export declare function createRenderableEditor(renderer: Renderer): MathEditorRenderable;
export {};
