/**
 * @hidden
 * @packageDocumentation
 */
import { DomRenderer, DomRenderedTree } from "../../standard-edition/editor/editor-support";
import { MathEditorRenderable } from "./editor-api";
import { ExtendedKeyboardShortcuts } from "./extended-keyboard-shortcuts";
import { FocusHandler } from "../../standard-edition/dom/focus-handler";
import { CursorSide } from "../../standard-edition/dom/mouse-selection";
import { IAdvice } from "../../standard-edition/accessibility/advices";
import { Point } from "../../standard-edition/model/utils";
import { AdviceListPopup } from "../../standard-edition/ui/advice-list-popup";
/**
 * Extends {@link MathEditorRenderable} and uses KaTeX as renderer.
 * Listen DOM mouse and keyboard events for expression manipulation.
 * Includes default semantic validator.
 * @category API
 */
export declare class MathEditor extends MathEditorRenderable {
    /**
     * @hidden
     */
    readonly keyboardShortcuts: ExtendedKeyboardShortcuts;
    /**
     * @hidden
     */
    readonly focusHandler: FocusHandler;
    /**
     * @hidden
     */
    protected renderedTree: DomRenderedTree;
    /**
     * @hidden
     */
    protected readonly renderer: DomRenderer;
    /**
     * @hidden
     */
    readonly element: HTMLElement;
    /**
     * @hidden
     */
    private locked;
    /**
     * @hidden
     */
    private readonly;
    /**
     * @hidden
     */
    private readonly mouseSelection;
    /**
     * @hidden
     */
    private readonly copyPasteHandler;
    /**
     * @hidden
     */
    readonly config: {
        selections: {
            frame: number;
            minDistance: number;
        };
        keyBindings: {
            navigation: boolean;
        };
    };
    constructor(element: HTMLElement, renderer: DomRenderer);
    /**
     * Returns whether user input locked or not.
     * @category API
     */
    get userInputLocked(): boolean;
    /**
     * Returns list of {@link Element} for a given node id.
     * @param nodeId
     * @category API
     */
    getDOMElement(nodeId: number): Element[];
    /**
     * @hidden
     * Returns validation callback implementing (demonstrative) reporting of semantic errors:
     * 1) checks for errors in given editor,
     * 2) if none found - renders no-errors HTML as descriptions,
     * 3) otherwise puts 'error-frame' class on appropriate nodes,
     * 4) renders error descriptions with 'error-description' class for each,
     * 5) connects frames with descriptions by putting 'error-highlight' class on corresponding description when hovering frame.
     *
     * Callback is suitable to be called either on a validation button pressed by user (eg. index.html) or programmatically
     * (eg. demo.html).
     *
     * Make sure to have styles for '.error-frame', '.error-description' and '.error-description.error-highlight'.
     */
    turnOnDefaultValidation(descriptionsElement: Element, customize?: {
        noErrorsHtml?: string;
        undefinedStateHtml?: string;
        descriptionElementProvider?: () => HTMLElement;
        frameRenderCallback?: (frame: HTMLElement) => void;
    }): () => void;
    /**
     * Validates current expression in the editor and highlights invalid nodes.
     * @category API
     */
    requestValidation(): void;
    /**
     * @hidden
     */
    get hintListPopup(): AdviceListPopup;
    /**
     * Locks user input.
     * @category API
     */
    lockUserInput(): void;
    /**
     * Unlocks user input.
     * @category API
     */
    unlockUserInput(): void;
    /**
     * Enables readonly mode for the editor.
     * @category API
     */
    setReadOnly(): void;
    /**
     * Disables readonly mode for the editor.
     * @category API
     */
    setReadWrite(): void;
    /**
     * Disables mouse interaction for the editor.
     * @category API
     */
    disableMouseInteraction(): void;
    /**
     * Enables mouse interaction for the editor.
     * @category API
     */
    enableMouseInteraction(): void;
    /**
     * Selects expression in the editor inside a given rectangle.
     * @param startPoint
     * @param endPoint
     * @category API
     */
    selectInRectangle(startPoint: Point, endPoint: Point): boolean;
    /**
     * Enables undo/redo for the editor.
     * @category API
     */
    enableUndoRedo(): void;
    /**
     * Disables undo/redo for the editor.
     * @category API
     */
    disableUndoRedo(): void;
    /**
     * Focuses the editor.
     * @category API
     */
    focus(): void;
    /**
     * Blurs the editor.
     * @category API
     */
    blur(): void;
    /**
     * Returns nearest node for a given point.
     * @param point
     * @category API
     */
    getNearestNode(point: Point): {
        nodeId: number;
        side: CursorSide;
    };
    /**
     * @hidden
     */
    showAdviceBox(advice: IAdvice[], showInLocked?: boolean): void;
    /**
     * Destroys the editor.
     * @category API
     */
    destroy(): void;
}
/**
 * Factory method that creates {@link MathEditor}.
 * @param element to mount math editor
 * @param multiLine true for multiline expression, for example system of equations or matrix.
 * @category API
 */
export declare function createEditor(element: HTMLElement, multiLine?: boolean): MathEditor;
