/**
 * @hidden
 * @packageDocumentation
 */
import { ConfirmRequiredOperation } from "../engine/internal-engine";
import { HorizontalDirection } from "../engine/utils";
import { UndoRedoEditor } from "../editor/undo-redo-editor";
export interface TreeUpdateCallbacks<NO, NU> {
    onUpdate?: (updateInfo: NU[], confirmation: ConfirmRequiredOperation) => void;
    onNotify?: (notifyInfo: NO[], confirmation: ConfirmRequiredOperation) => void;
}
/**
 * Event that notifies about changing state of the editor. See {@link MathEditorAPI.addOnUpdate} and {@link MathEditorAPI.removeOnUpdate}.
 * @category API
 */
export declare enum NotifyUpdateObject {
    tree = "tree",
    cursor = "cursor",
    selection = "selection"
}
/**
 * Event that doesn't count as changing state of the editor.  See {@link MathEditorAPI.addOnNotify} and {@link MathEditorAPI.removeOnNotify}.
 * @category API
 */
export declare enum NotifyObject {
    copy = "copy",
    inputTree = "inputTree",
    nonInputTree = "nonInputTree",
    assistPopupOpen = "assistPopupOpen",
    assistPopupClose = "assistPopupClose",
    assistAdviceHover = "assistAdviceHover",
    assistApply = "assistApply",
    failedNavigationLeft = "failedNavigationLeft",
    failedNavigationRight = "failedNavigationRight",
    failedNavigationHome = "failedNavigationHome",
    failedNavigationEnd = "failedNavigationEnd",
    failedNavigationUp = "failedNavigationUp",
    failedNavigationDown = "failedNavigationDown",
    failedSelectionLeft = "failedSelectionLeft",
    failedSelectionRight = "failedSelectionRight",
    failedSelectionHome = "failedSelectionHome",
    failedSelectionEnd = "failedSelectionEnd",
    failedSelectionUp = "failedSelectionUp",
    failedSelectionDown = "failedSelectionDown",
    failedBackspace = "failedBackspace",
    failedDelete = "failedDelete",
    empty = "empty",
    focus = "focus",
    blur = "blur",
    initialize = "initialize",
    decoration = "decoration",
    editableCell = "editableCell",
    asksConfirm = "asksConfirm"
}
export interface IUpdateContainer<TemplateObject> {
    notify: (type: TemplateObject[] | TemplateObject, confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean) => void;
    readonly isPaused: boolean;
    confirmOperation: ConfirmRequiredOperation;
    pauseSync(): void;
    resumeSync(insideOnUpdate: boolean): void;
    add(cb: (info: TemplateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    remove(cb: (info: TemplateObject[], confirmation: ConfirmRequiredOperation) => void): void;
}
/**
 * Generic class used to handle several types of update information in the same way
 */
export declare class UpdateContainer<TemplateObject> implements IUpdateContainer<TemplateObject> {
    private _confirmOperation;
    protected pauseOnUpdate: boolean;
    protected pausedObjects: TemplateObject[];
    private innerCallbacks;
    readonly notify: (type: TemplateObject[] | TemplateObject, confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean) => void;
    constructor(externalCallback?: (info: TemplateObject[], confirmation: ConfirmRequiredOperation) => void);
    pauseSync(): void;
    resumeSync(insideOnUpdate: boolean): void;
    add(cb: (info: TemplateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    remove(cb: (info: TemplateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    get isPaused(): boolean;
    get confirmOperation(): ConfirmRequiredOperation;
    set confirmOperation(value: ConfirmRequiredOperation);
}
export interface UpdateListenersHolder {
    addOnUpdate(cb: (updateInfo: NotifyUpdateObject[], confirmation: ConfirmRequiredOperation) => void): void;
    removeOnUpdate(cb: (updateInfo: NotifyUpdateObject[], confirmation: ConfirmRequiredOperation) => void): void;
}
export declare abstract class AbstractDomBinderNotifyUpdater<NO, NU> {
    protected notifyContainer: IUpdateContainer<NO>;
    protected updateContainer: IUpdateContainer<NU>;
    private readonly editor;
    constructor(editor: UndoRedoEditor, externalCallbacks?: TreeUpdateCallbacks<NO, NU>);
    addOnUpdate(cb: (updateInfo: NU[], confirmation: ConfirmRequiredOperation) => void): void;
    removeOnUpdate(cb: (updateInfo: NU[], confirmation: ConfirmRequiredOperation) => void): void;
    addOnNotify(cb: (updateInfo: NO[]) => void): void;
    removeOnNotify(cb: (updateInfo: NO[]) => void): void;
    pauseSync(): void;
    resumeSync(insideOnUpdate: boolean): void;
    notifyOnUpdate(updateType: NU | NU[], confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean): void;
    notify(updateType: NO | NO[], confirmation?: ConfirmRequiredOperation, calledInsideOnUpdate?: boolean): void;
    isPaused(): boolean;
    getCurrentConfirmOperation(): ConfirmRequiredOperation;
    resetConfirmOperation(): void;
    isOperationConfirmed(direction: HorizontalDirection): boolean;
}
export declare class DomBinderNotifyUpdater extends AbstractDomBinderNotifyUpdater<NotifyObject, NotifyUpdateObject> implements UpdateListenersHolder {
}
